@@ -92,9 +92,11 @@ class SaveStateSWACallback(CallbackInterface, metaclass=SuperclassDocstring):
         self.best_epoch: int = 0
         self.best_iter: int = 0
 
-    def _extract_swa_state(self,
-                           model: PyTorchEstimators,
-                           swa_model: AveragedModel):
+    def _extract_swa_state(
+        self,
+        model: PyTorchEstimators,
+        swa_model: AveragedModel
+    ):
         r"""
         Méthode privée qui permet d'extraire les poids du modèle SWA et les 
         renommer pour correspondre aux noms des poids du modèle.
@@ -113,58 +115,61 @@ class SaveStateSWACallback(CallbackInterface, metaclass=SuperclassDocstring):
         )
         return swa_weights
 
-    def end_eval(self,
-                 model: PyTorchEstimators,
-                 data: List[Dict[str, Tensor]],
-                 state: FitState,
-                 control: FitControl,
-                 res_loss: float,
-                 swa_model: AveragedModel) -> None:
-            if (
-                    res_loss < self.old_res_loss 
-                    if self.minimize else 
-                    res_loss > self.old_res_loss
-            ):
-                if self.verbose:
-                    print(
-                        "\n[SWA] Gain de performance époque/itération : "
-                        f"{state.n_epoch_swa+1}/{state.n_iter_swa+1} ;\n"
-                        f"Gain de performance {state.loss_name} :\n"
-                        f"{self.old_res_loss} -> {res_loss}.\n",
-                        flush=True
-                    )
-                self.iter_stop = 0
-                self.old_res_loss = res_loss
-                self.best_epoch = state.n_epoch_swa
-                self.best_iter = state.n_iter_swa
-                swa_model.update_parameters(model)
-                if self.path_save is None:
-                    self.state_dict = self._extract_swa_state(model, swa_model)
-                else:
-                    model.save_weights(
-                        f"{self.path_save}/{state.id_process}.model",
-                        state_dict=self._extract_swa_state(model, swa_model)
-                    )
-            elif (
-                    self.early_stopping_rounds is not None and 
-                    (
-                        res_loss >= self.old_res_loss 
-                        if self.minimize else
-                        res_loss <= self.old_res_loss
-                    )
-            ):
-                self.iter_stop += 1
-                if self.iter_stop >= self.early_stopping_rounds:
-                    control.training_stop = True
+    def end_eval(
+        self,
+        model: PyTorchEstimators,
+        data: List[Dict[str, Tensor]],
+        state: FitState,
+        control: FitControl,
+        res_loss: float,
+        swa_model: AveragedModel) -> None:
+        if (
+                res_loss < self.old_res_loss 
+                if self.minimize else 
+                res_loss > self.old_res_loss
+        ):
+            if self.verbose:
+                print(
+                    "\n[SWA] Gain de performance époque/itération : "
+                    f"{state.n_epoch_swa+1}/{state.n_iter_swa+1} ;\n"
+                    f"Gain de performance {state.loss_name} :\n"
+                    f"{self.old_res_loss} -> {res_loss}.\n",
+                    flush=True
+                )
+            self.iter_stop = 0
+            self.old_res_loss = res_loss
+            self.best_epoch = state.n_epoch_swa
+            self.best_iter = state.n_iter_swa
+            swa_model.update_parameters(model)
+            if self.path_save is None:
+                self.state_dict = self._extract_swa_state(model, swa_model)
+            else:
+                model.save_weights(
+                    f"{self.path_save}/{state.id_process}.model",
+                    state_dict=self._extract_swa_state(model, swa_model)
+                )
+        elif (
+                self.early_stopping_rounds is not None and 
+                (
+                    res_loss >= self.old_res_loss 
+                    if self.minimize else
+                    res_loss <= self.old_res_loss
+                )
+        ):
+            self.iter_stop += 1
+            if self.iter_stop >= self.early_stopping_rounds:
+                control.training_stop = True
 
-    def end_train(self,
-                  model: PyTorchEstimators,
-                  data: List[Dict[str, Tensor]],
-                  state: FitState,
-                  control: FitControl,
-                  res_loss: float,
-                  swa_model: AveragedModel,
-                  eval_dataset: Optional[DataLoader]) -> None:
+    def end_train(
+        self,
+        model: PyTorchEstimators,
+        data: List[Dict[str, Tensor]],
+        state: FitState,
+        control: FitControl,
+        res_loss: float,
+        swa_model: AveragedModel,
+        eval_dataset: Optional[DataLoader]
+    ) -> None:
         if self.verbose:
             print(
                 f"\n[SWA] Meilleure époque/iteration : {self.best_epoch+1}/"
@@ -324,132 +329,12 @@ class StochasticWeightAveraging:
     minimize: bool = True
     verbose: bool = True
 
-    def _decomp_subset(self,
-                       data: Union[Dataset, DataLoader, List[Dict[str, Any]]],
-                       ratio: float,
-                       stratified: bool=False,
-                       field_target: str='target',
-                       random_state: int=42):
-        r"""
-        Méthode interne qui découpe en deux datasets en fonction d'un taux de 
-        recouvrement des observations.
-
-        Parameters
-        ----------
-        data : Union[Dataset, DataLoader, List[Dict[str, Any]]]
-            Dataset ou list de dictionnaire des observations.
-        ratio : float
-            Taux de recouvrement des observations entre les deux datasets.
-        stratified : bool (=False)
-            Permet de stratifier la répartition en fonction des classes de la 
-            target (ne fonctionne que si il s'agit d'un classifieur).
-        field_target : str (='target')
-            Précise le champs target si il y a une stratification (sinon non 
-            utilisé).
-        random_state : int (=42)
-            Random state de la répartition des observations.
-
-        Returns
-        -------
-        DatasetTrain : Union[Datast, Dataset, List[Dict[str, Tensor]]]
-            Dataset utilisé pour la phase d'entrainement
-        """
-        if ratio < 0 or ratio > 1:
-            raise AttributeError(
-                "swa_inner_ratio doit être compris entre 0 et 1 "
-                f"(inclus). Ici il vaut {ratio} !"
-            )
-        if stratified and self._estimator_type != 'classifier':
-            raise AttributeError(
-                "Pour l'approche stratifiée de l'intersection jointe il faut "
-                "que le modèle soit un classifier, or il s'agit d'une "
-                f"{self._estimator_type} !"
-            )
-        if isinstance(data, DataLoader):
-            is_dataloader = True
-            dl_args = {
-                kk: vv for kk, vv in data.__dict__.itmes()
-                if not kk.startswith('_')
-                and not kk in (
-                    'dataset', 'batch_size', 'drop_last', 'sampler'
-                )
-            }
-            data = data.dataset
-        else:
-            is_dataloader = False
-        size = len(data)
-        idx_ = np.arange(size)
-        y = None
-        if ratio < 1:
-            if stratified:
-                split = StratifiedShuffleSplit(
-                    n_splits=1,
-                    train_size=int((0.5+ratio*0.5)*size),
-                    random_state=random_state
-                )
-                y = list(map(lambda x: x[field_target], data))
-            else:
-                split = ShuffleSplit(
-                    n_splits=1,
-                    train_size=int((0.5+ratio*0.5)*size),
-                    random_state=random_state
-                )
-            idx_over, idx_rest = next(split.split(idx_, y))
-        else:
-            idx_over, idx_rest = idx_, np.array([])
-        idx_train = np.r_[
-            idx_rest[:int(idx_rest.shape[0]/2)],
-            idx_over
-        ]
-        idx_swa = np.r_[
-            idx_rest[int(idx_rest.shape[0]/2):],
-            idx_over
-        ]
-        if isinstance(data, Dataset):
-            return Subset(data, idx_train), Subset(data, idx_swa)
-        elif is_dataloader:
-            return (
-                DataLoader(dataset=Subset(data, idx_train), **dl_args),
-                DataLoader(dataset=Subset(data, idx_swa), **dl_args)
-            )
-            
-        else:
-            return (
-                [data[ii] for ii in idx_train],
-                [data[ii] for ii in idx_swa]
-            )
-
     def copy(self):
         r"""
         Créer une copie de l'objet.
         """
         return deepcopy(self)
 
-    def cut_dataset(self,
-                    dataset: Union[List[Dict[str, Any]], Dataset, DataLoader],
-                    field_target: str='target'):
-        r"""
-        Méthode permettant le découpage entre deux datasets d'entraînement et 
-        le dataset pour le processus SWA.
-
-        Parameters
-        ----------
-        dataset : Union[List[Dict[str, Any]], Dataset, DataLoader]
-            Dataset ou list de dictionnaire des observations.
-        field_target : str (='target')
-            Précise le champs target si il y a une stratification (sinon non 
-            utilisé).
-        """
-        self.field_target = field_target
-        X, self.X_swa = self._decomp_subset(
-            dataset,
-            ratio=self.inner_ratio,
-            stratified=self.stratified,
-            field_target=field_target,
-            random_state=self.random_state
-        )
-        return X
-
     def init(self,
              model: PyTorchEstimators,
              res_loss: float):
@@ -832,3 +717,102 @@ class StochasticWeightAveraging:
             swa_model=swa_model,
             eval_dataset=eval_dataset
         )
+
+def cut_dataset(
+    model: PyTorchEstimators,
+    data: Union[Dataset, DataLoader, List[Dict[str, Any]]],
+    *,
+    ratio: float=0.5,
+    stratified: bool=False,
+    target_field: str='target',
+    random_state: int=42
+):
+    r"""
+    Fonction qui découpe un dataset en deux datasets en fonction d'un taux de 
+    recouvrement des observations.
+
+    Parameters
+    ----------
+    model: PyTorchEstimators
+        Modèle d'apprentissage PyTorch.
+    data : Union[Dataset, DataLoader, List[Dict[str, Any]]]
+        Dataset ou list de dictionnaire des observations.
+    ratio : float (=0.5)
+        Taux de recouvrement des observations entre les deux datasets.
+    stratified : bool (=False)
+        Permet de stratifier la répartition en fonction des classes de la 
+        target (ne fonctionne que si il s'agit d'un classifieur).
+    target_field : str (='target')
+        Précise le champs target si il y a une stratification (sinon non 
+        utilisé).
+    random_state : int (=42)
+        Random state de la répartition des observations.
+
+    Returns
+    -------
+    DatasetTrain : Union[Datast, Dataset, List[Dict[str, Tensor]]]
+        Dataset utilisé pour la phase d'entrainement
+    """
+    if ratio < 0 or ratio > 1:
+        raise AttributeError(
+            "swa_inner_ratio doit être compris entre 0 et 1 "
+            f"(inclus). Ici il vaut {ratio} !"
+        )
+    if stratified and model._estimator_type != 'classifier':
+        raise AttributeError(
+            "Pour l'approche stratifiée de l'intersection jointe il faut "
+            "que le modèle soit un classifier, or il s'agit d'une "
+            f"{model._estimator_type} !"
+        )
+    if isinstance(data, DataLoader):
+        is_dataloader = True
+        dl_args = {
+            kk: vv for kk, vv in data.__dict__.items()
+            if not kk.startswith('_')
+            and not kk in ('dataset', 'batch_sampler')
+        }
+        data = data.dataset
+    else:
+        is_dataloader = False
+    size = len(data)
+    idx_ = np.arange(size)
+    y = None
+    if ratio < 1 and ratio > 0:
+        if stratified:
+            split = StratifiedShuffleSplit(
+                n_splits=1,
+                train_size=int(ratio*size),
+                random_state=random_state
+            )
+            y = list(map(lambda x: x[target_field], data))
+        else:
+            split = ShuffleSplit(
+                n_splits=1,
+                train_size=int(ratio*size),
+                random_state=random_state
+            )
+        idx_over, idx_rest = next(split.split(idx_, y))
+    elif ratio == 1:
+        idx_over, idx_rest = idx_, np.array([])
+    elif ratio == 0:
+        idx_over, idx_rest = np.array([]), idx_
+    idx_train = np.r_[
+        idx_rest[:int(idx_rest.shape[0]//2)],
+        idx_over
+    ]
+    idx_swa = np.r_[
+        idx_rest[int(idx_rest.shape[0]//2):],
+        idx_over
+    ]
+    if is_dataloader:
+        return (
+            DataLoader(dataset=Subset(data, idx_train), **dl_args),
+            DataLoader(dataset=Subset(data, idx_swa), **dl_args)
+        )
+    elif isinstance(data, Dataset):
+        return Subset(data, idx_train), Subset(data, idx_swa)
+    else:
+        return (
+            [data[ii] for ii in idx_train],
+            [data[ii] for ii in idx_swa]
+        )
