@@ -8,18 +8,22 @@ Created on Thu Oct 31 09:42:49 2019
 @author: Cyrile Delestre
 """
 from typing import Union, Optional, Tuple, List, Type, Dict, Any
+from warnings import warn
 
 import numpy as np
 from pandas.core.frame import DataFrame
 
-from torch import Tensor, from_numpy
+import torch
 
 from dstk.utils import shape_list
 from dstk.utils.deprecated import deprecated
 from dstk.utils.errors import CheckError
 
-def check_tensor(array: Union[Tensor, np.ndarray, DataFrame, List, Tuple, int, float],
-                 dtype: Optional[Type]=None):
+
+def check_tensor(
+        array: Union[torch.Tensor, np.ndarray, DataFrame, List, Tuple, int, float],
+        dtype: Optional[Type]=None
+    ):
     r"""
     Fonction vérifiant si le type de l'entrée est bien transformable en Tensor 
     PyTorch et, si c'est le cas, transformation de ce dernier en suivant le 
@@ -39,36 +43,37 @@ def check_tensor(array: Union[Tensor, np.ndarray, DataFrame, List, Tuple, int, f
     tensor : Tensor
         array transformé en tensor au type dtype
     """
-    if isinstance(array, Tensor):
+    if isinstance(array, torch.Tensor):
         return array if dtype is None else array.to(dtype)
     elif isinstance(array, np.ndarray):
         return (
-            from_numpy(array)
+            torch.from_numpy(array)
             if dtype is None
-            else from_numpy(array).to(dtype)
+            else torch.from_numpy(array).to(dtype)
         )
     elif isinstance(array, DataFrame):
         return (
-            from_numpy(array.values)
+            torch.from_numpy(array.values)
             if dtype is None
-            else from_numpy(array.values).to(dtype)
+            else torch.from_numpy(array.values).to(dtype)
         )
     elif isinstance(array, (list, tuple,)):
         return (
-            from_numpy(np.array(array))
+            torch.from_numpy(np.array(array))
             if dtype is None
-            else from_numpy(np.array(array)).to(dtype)
+            else torch.from_numpy(np.array(array)).to(dtype)
         )
     elif isinstance(array, (int, float, np.integer, np.floating,)):
         return (
-            Tensor(np.array([array]))
+            torch.Tensor(np.array([array]))
             if dtype is None
-            else Tensor(np.array([array])).to(dtype)
+            else torch.Tensor(np.array([array])).to(dtype)
         )
     else:
         raise CheckError
 
-def check_tensor_dict(data: Dict[str, Any]):
+
+def check_tensor_dict(data: Dict[str, Any], add_dim_batch: bool=False):
     r"""
     Fonction qui va transformer les éléments d'un dictionnaire en PyTorch 
     Tensor si c'est faisable.
@@ -77,44 +82,28 @@ def check_tensor_dict(data: Dict[str, Any]):
     ----------
     data : Dict[str, Any]
         dictionnaire pouvant contenir ce que l'on souhaite.
-    
+    add_dim_batch : bool (=False)
+        ajoute la dimension du batch si elle n'est pas présente.
+
     Returns
     -------
     data : Dict[str, Tensor]
         dictionnaire contenant des Tensor
     """
     def prepar_data(x):
-        if isinstance(data[x], (list, tuple,)):
-            if len(shape_list(data[x])) >= 2:
-                return check_tensor(data[x])
-            else:
-                return check_tensor([data[x]])
-        elif isinstance(data[x], (int, float,)):
-            return check_tensor(data[x])
-        elif isinstance(data[x], np.ndarray):
-            if len(data[x].shape) >= 2:
-                return check_tensor(data[x])
-            else:
-                return check_tensor(data[x][np.newaxis, :])
-        elif isinstance(data[x], DataFrame):
-            if len(data[x].shape) >= 2:
-                return check_tensor(data[x])
-            else:
-                return check_tensor(data[x].values[np.newaxis, :])
-        elif isinstance(data[x], Tensor):
-            if data[x].dim() >= 2:
-                return data[x]
-            else:
-                return data[x].unsqueeze(0)
+        data_ = check_tensor(data[x])
+        return data_.unsqueeze(0) if add_dim_batch else data_
     return {
         ii: prepar_data(ii)
         for ii in data 
         if isinstance(
             data[ii],
-            (Tensor, np.ndarray, list, tuple, DataFrame, int, float,)
+            (torch.Tensor, np.ndarray, list, tuple, DataFrame, int, float,
+             np.integer, np.floating,)
         )
     }
 
+
 @deprecated("Fonction plus utilisé, sera supprimée dans les MAJ à venir !")
 def detach(array):
     r"""
@@ -134,7 +123,7 @@ def detach(array):
     Préservation de la structure de départ avec les tensors détachés du graph
     """
     if isinstance(array, (list, tuple)):
-        if isinstance(array[0], Tensor):
+        if isinstance(array[0], torch.Tensor):
             res = []
             for aa in array:
                 if aa.requires_grad:
@@ -145,7 +134,7 @@ def detach(array):
                 return tuple(res)
             return res
         elif isinstance(array[0], (list, tuple)):
-            if isinstance(array[0][0], Tensor):
+            if isinstance(array[0][0], torch.Tensor):
                 res = []
                 for aa in array:
                     res_int = []
@@ -180,7 +169,7 @@ def detach(array):
     elif isinstance(array, dict):
         items = list(array.items)
         if isinstance(items[0][0], (list, tuple)):
-            if isinstance(items[0][0][0], Tensor):
+            if isinstance(items[0][0][0], torch.Tensor):
                 res = []
                 for key, aa in items:
                     res_int = []
@@ -204,7 +193,7 @@ def detach(array):
                         type(items[0][0][0])
                     )
                 )
-        elif isinstance(items[0][0], Tensor):
+        elif isinstance(items[0][0], torch.Tensor):
             res = []
             for key, aa in items:
                 if aa.requires_grad:
@@ -218,7 +207,7 @@ def detach(array):
                 "qu'il s'agit d'un type {} de {}."
                 .format(type(array), type(items[0][0]))
             )
-    elif isinstance(array, Tensor):
+    elif isinstance(array, torch.Tensor):
         if array.requires_grad:
             return array.detach()
         return array
@@ -229,11 +218,14 @@ def detach(array):
             .foramt(type(array))
         )
 
-def collate_zeros_padding(sample: List[Dict[str, Union[Tensor, np.ndarray]]],
-                          fields_series: List[Union[List, Tuple]],
-                          padding_value: float=0,
-                          torch_type: bool=True,
-                          batch_first: bool=True):
+
+def collate_zeros_padding(
+    sample: List[Dict[str, Union[torch.Tensor, np.ndarray]]],
+    fields_series: List[Union[List, Tuple]],
+    padding_value: float=0,
+    torch_type: bool=True,
+    batch_first: bool=True
+):
     r"""
     Fonction de padding générique pour des séquences de différente longueur.
     
@@ -332,7 +324,7 @@ def collate_zeros_padding(sample: List[Dict[str, Union[Tensor, np.ndarray]]],
                     f"{len(sample[0][{arg}].shape)} !"
                 )
 
-            if isinstance(sample[0][arg], Tensor):
+            if isinstance(sample[0][arg], torch.Tensor):
                 dtype = sample[0][arg].numpy().dtype
             else:
                 dtype = sample[0][arg].dtype
@@ -345,10 +337,9 @@ def collate_zeros_padding(sample: List[Dict[str, Union[Tensor, np.ndarray]]],
                 else:
                     samp_len = samp[key_len]
                 tensor[obj_slice(ii, samp_len)] = samp[arg]
-
         else:
-            if isinstance(sample[0][arg], (np.ndarray, Tensor)):
-                if isinstance(sample[0][arg], Tensor):
+            if isinstance(sample[0][arg], (np.ndarray, torch.Tensor)):
+                if isinstance(sample[0][arg], torch.Tensor):
                     dtype = sample[0][arg].numpy().dtype
                 else:
                     dtype = sample[0][arg].dtype
@@ -387,5 +378,69 @@ def collate_zeros_padding(sample: List[Dict[str, Union[Tensor, np.ndarray]]],
             for ii, samp in enumerate(sample):
                 tensor[obj_slice(ii)] = samp[arg]
 
-        batch[arg] = from_numpy(tensor) if torch_type else tensor
+        batch[arg] = torch.from_numpy(tensor) if torch_type else tensor
     return batch
+
+
+def make_mask(
+        in_features: int,
+        out_features: int,
+        n_channel: int,
+        shift: int,
+        kernel_size: int,
+        repeat: Optional[Union[List, Tuple]]=None
+) -> torch.BoolTensor:
+    r"""
+    Fonction de création de masque permettant de personaliser la topologie 
+    des réseaux linéaires pour être autoregressif.
+    
+    Parameters
+    ----------
+    in_features : int
+        taille en entré du réseau
+    out_features : int
+        taille en sortie du réseau
+    n_channel : int
+        nombre de neuronnes voyant les mêmes entrées
+    shift : int
+        décalage incrémentale sur chaques neuronnes
+    kernel_size : int
+        entrée contigue en entré du neuronne
+    repeat : Optional[Union[List, Tuple]] (=None)
+        tuple ou liste permettant de constituer les connexions (utile si 
+        entrés multivariées).
+    
+    Returns
+    -------
+    mask : BoolTensor
+        masque de dimension (out_feature x repeat[0]), 
+        (in_features x repeat[1])
+    
+    Examples
+    --------
+    >>> import torch
+    >>> from dstk.pytorch import make_mask
+    >>> 
+    >>> make_mask(6,6,1,0,2, repeat=(1,2)).type(torch.int8)
+    tensor([[1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0]], dtype=torch.int8)
+    >>> 
+    >>> make_mask(6,6,2,2,2, repeat=(1,2)).type(torch.int8)
+    tensor([[1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0],
+            [0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0],
+            [0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0],
+            [0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1],
+            [0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1]], dtype=torch.int8)
+    """
+    mask = torch.zeros(out_features, in_features, dtype=torch.bool)
+    for ii in range(0, out_features, n_channel):
+        ii_shift = int(ii/n_channel)*shift
+        mask[ii:ii+n_channel, ii_shift:ii_shift+kernel_size] = True
+    if repeat:
+        return mask.repeat(repeat)
+    return mask
